import QtQuick 2.0
import QtQuick.XmlListModel 2.0

import Ubuntu.Components 0.1
import Ubuntu.Components.ListItems 0.1 as ListItems
import Ubuntu.Components.Popups 0.1

import "databasemodule.js" as DB

Page {
    id: pageItself

    signal feedSelected(string selectedFeed)
    signal feedsReloaded
    signal feedDeleted(string feedLink)

    ListView {
        id: feedList

        function reloadFeeds() {
            feedListModel.clear()
            var feeds = DB.loadFeeds()
//            var feeds = DB.dbResult
            for (var i = 0; i < feeds.length; i++) {
                feedListModel.append({"link":feeds.item(i).link,
                                         "displayName":feeds.item(i).displayName,
                                         "siteLink":feeds.item(i).siteLink,
                                         "description":feeds.item(i).description})
            }
        }

        function infoLoadedByModel(url, data) {
            DB.updateFeedWithInfo(url, data)
            reloadFeeds()
        }

        function loadFailed(url) {
            console.log("LOAD FAILED FOR", url)
            DB.deleteFeed(url)
            feedList.reloadFeeds()
        }

        anchors.fill: parent
        model: ListModel {
            id: feedListModel
        }

        header: ListItems.Subtitled {
            text: i18n.tr("All news")
            subText: i18n.tr("View news from all channels")
            progression: true
            visible: feedListModel.count != 0

            onClicked: {
                pageItself.feedSelected("") // will show all feeds.
            }
        } // header

        delegate: ListItems.Subtitled {
            text: (model.displayName === "")? model.link : model.displayName
            subText: model.description
            progression: true

            onPressAndHold: {
                feedList.currentIndex = model.index
                PopupUtils.open(popoverComponent, feedList)
            }

            onClicked: {
                console.log("Show news in feed:", model.link)
                pageItself.feedSelected(model.link)
            }
        } // delegate

        Component.onCompleted: {
            reloadFeeds()
            helperModel.infoLoaded.connect(infoLoadedByModel)
            helperModel.loadFailed.connect(loadFailed)
        }

    } // ListView

    /* Thin wrapper around XMLHttpRequest.
     * 0 stage. Loading raw XML data from network.
     */
    QtObject {
        id: thinXmlHttpRequestWraper

        signal xmlLoaded(string xml)

        function loadXml(xmlUrl) {
            var doc = new XMLHttpRequest();
            doc.onreadystatechange = function() {
                if (doc.readyState == XMLHttpRequest.DONE) {
                    if (doc.status == 200 ) {
                        xmlLoaded(doc.responseText)
                    } else {
                        console.log("XMLHttpRequest ERROR", doc.status, doc.statusText)
                        feedList.loadFailed(xmlUrl) // mb need to replace with signal, as in xmllistmodel.
                    }
                }
            }

            doc.open("GET", xmlUrl);
            doc.send();
        }

        onXmlLoaded: {
            helperModel.xml = xml
            // Another functions, such as saving in database and so on.
        }
    } // QtObject

    /* First stage - need to gain information about feed from XML
     * and save it to database. Next step - loading entries.
     */
    XmlListModel {
        id: helperModel

        property string currentUrl: "" // TODO Move to another place later.

        // signal infoLoaded(string url, string title, string siteLink, string description)
        signal infoLoaded(string url, var data)
        signal loadFailed(string url)

        query: "/rss/channel"

        XmlRole { name: "displayName"; query: "title/string()" }
        XmlRole { name: "siteLink"; query: "link/string()" }
        XmlRole { name: "description"; query: "description/string()" }

        onStatusChanged: {
            var firstItem
            switch(status)
            {
            case XmlListModel.Error:
                console.log("XmlListModel", "ERROR", errorString())
                loadFailed(source)
                break
            case XmlListModel.Ready:
                console.log("XmlListModel", "READY", "COUNT", count)
                if (count === 1) {
                    firstItem = helperModel.get(0)
                    infoLoaded(currentUrl, firstItem)
                    xmlModelToParseItems.startEntriesParsing(currentUrl, xml)
                    currentUrl = "" // TODO mb no need
                } else { // if (count === 0 ) {
                    loadFailed(currentUrl)
                }
                break
            case XmlListModel.Loading:
                console.log("XmlListModel", "LOADING")
                break
            case XmlListModel.Null:
                console.log("XmlListModel", "NULL")
                break
            }
        }
    } // XmlListModel

    /* Second stage - need to parse XML and get all items from it.
     * This items will be displayed by "NewsListPage.qml"
     *
     * Very useful http://www.w3schools.com/xpath/xpath_functions.asp
     */
    XmlListModel {
        id: xmlModelToParseItems

        property string curUrl: ""

        function startEntriesParsing(url, strXml) {
            xmlModelToParseItems.curUrl = url
            xmlModelToParseItems.xml = strXml
        }

        query: "/rss/channel/item"

        XmlRole { name: "title"; query: "title/string()" }
        XmlRole { name: "pubDate"; query: "pubDate/string()" }
        XmlRole { name: "description"; query: "description/string()" }
        XmlRole { name: "content"; query: "*[name()='content:encoded']/string()" }

        onStatusChanged: {
            var entry
            switch(status)
            {
            case XmlListModel.Error:
                console.log("xmlModelToParseItems", "ERROR", errorString())
                break
            case XmlListModel.Ready:
            {
                console.log("xmlModelToParseItems", "READY", "COUNT", count)
                for (var i = 0; i < count; i++) {
                    entry = xmlModelToParseItems.get(i)
                    DB.addEntry(curUrl,  Qt.md5(entry.title) /*TODO MORE DATA IN HASH*/,
                                entry)
                    //                    DB.addEntry(curUrl, entry.title, entry.pubDate, Qt.md5(entry.title) /*TODO MORE DATA IN HASH*/,
                    //                                entry.description, entry.content)
                }
                feedsReloaded()
            }
            break
            case XmlListModel.Loading:
                console.log("xmlModelToParseItems", "LOADING")
                break
            case XmlListModel.Null:
                console.log("xmlModelToParseItems", "NULL")
                break
            }
        }
    } // XmlListModel

    Component {
        id: popoverComponent

        /* This new popover works like context menu.
         * Actions can be reused in toolbar.
         */
        ActionSelectionPopover {
            id: popover
            actions: ActionList {
                id: popoverActionsList
                Action {
                    id: nameActionUpdate
                    text: i18n.tr("Update")
                    onTriggered: {
                        PopupUtils.close(popover)
                        var feedUrl = feedListModel.get(feedList.currentIndex).link
                        console.log("UPDATE STARTED FOR FEED:", feedUrl)
                        helperModel.currentUrl = feedUrl
                        thinXmlHttpRequestWraper.loadXml(feedUrl)
                    }
                }

                Action {
                    text: i18n.tr("Edit")
                    onTriggered: {
                        PopupUtils.close(popover)
                        console.log("TODO Edit")
                        PopupUtils.open(popoverSheetComponent, btnAppendFeed, {"isForEdit":true, "itemData" : feedListModel.get(feedList.currentIndex)})
                    }
                }

                Action {
                    text: i18n.tr("Open site")
                    onTriggered: {
                        PopupUtils.close(popover)
                        Qt.openUrlExternally(feedListModel.get(feedList.currentIndex).siteLink)
                    }
                }

                Action {
                    text: i18n.tr("Remove")
                    onTriggered: {
                        PopupUtils.close(popover)
                        var link = feedListModel.get(feedList.currentIndex).link
                        var dbResult = DB.deleteFeed(link)
                        if (dbResult.rowsAffected > 0) {
                            DB.removeEntriesOfFeed(link)
                            feedListModel.remove(feedList.currentIndex)
                        }
                        feedDeleted(link)
                    }
                }

                Action {
                    text: i18n.tr("Remove all")
                    onTriggered: {
                        PopupUtils.close(popover)
                        DB.dropTables()
                        feedListModel.clear()
                        feedsReloaded()
                    }
                }
            } // ActionList
        } // ActionSelectionPopover
    } // Popover component

    /* Modal sheet for appending feeds.
     * Demo of this: https://ubuntu.mybalsamiq.com/projects/ubuntuphonecoreapps/RSS_reader_step1.1
     */
    Component {
        id: popoverSheetComponent

        ComposerSheet {
            id: appendSheet

            property bool isForEdit: false
            property var itemData: null
            property alias showOnMainScreen: swShowOnMainscreen.checked

            Component.onCompleted: {
                // tfFeedUrl.focus()
                if (itemData)
                    tfFeedUrl.text = itemData.displayName
                tfFeedUrl.forceActiveFocus()
                tfFeedUrl.cursorPosition = tfFeedUrl.text.length
                console.log("IS FOR EDIT", isForEdit)
            }

            onConfirmClicked: {

                if (isForEdit) {
                    if (tfFeedUrl.text == "") {
                        console.log("EDIT TEXT empty, will use old value")
                    } else {
                        itemData.displayName = tfFeedUrl.text
                    }
                    DB.updateFeedWithInfo(itemData.link, itemData)
                    feedList.reloadFeeds() // TODO Temporary and slow reloading.
                } else {
                    /* Default value for faster development :)
                     * URL in comment - my Russian site, on which I tested program.
                     */
                    var feedUrl = (tfFeedUrl.text == "")? "http://news.yahoo.com/rss/" : tfFeedUrl.text // "http://qtrosapps.hostenko.com/feed/"
                    var obj = {"link":feedUrl}
                    DB.addFeed(obj)
                    if (DB.dbResult.exist) {
                        return
                    }
                    feedList.reloadFeeds() // TODO Temporary and slow reloading.
                    helperModel.currentUrl = feedUrl
                    thinXmlHttpRequestWraper.loadXml(feedUrl)
                }
            }

            onCancelClicked: {
                console.log("Canceled")
            }

            contentsWidth: feedList.width - units.gu(7)
            // contentsHeight: units.gu(11) // TODO BUG - doesn't works
            title: isForEdit? i18n.tr("Edit feed") : i18n.tr("Append feed")

            Column {
                spacing: units.gu(1)
                width: units.gu(30)
                anchors.centerIn: parent

                TextField {
                    id: tfFeedUrl
                    anchors.horizontalCenter: parent.horizontalCenter
                    width: parent.width
                    placeholderText: isForEdit? i18n.tr("Enter new name here...") : i18n.tr("Enter url here...")
                }

                Item {
                    width: parent.width
                    height: units.gu(5)

                    Label {
                        anchors {
                            left: parent.left
                            verticalCenter: parent.verticalCenter
                            leftMargin: units.gu(1)
                        }
                        text: i18n.tr("Show in inbox")
                    }

                    Switch {
                        id: swShowOnMainscreen
                        checked: true
                        anchors {
                            right: parent.right
                            verticalCenter: parent.verticalCenter
                            rightMargin: units.gu(1)
                        }
                    }
                }
            } // Column
        } // ComposerSheet
    } // Component popoverSheetComponent

    Label {
        id: lblEmptyBase
        text: i18n.tr("There are no RSS feeds to show")
        anchors.centerIn: parent
        visible: feedListModel.count == 0
    }

    ActivityIndicator {
        visible: helperModel.status == XmlListModel.Loading // TODO Need fix this
        running: visible

        anchors.centerIn: parent
    }

    Row {

        anchors {
            bottom: parent.bottom
            horizontalCenter: parent.horizontalCenter
            margins: units.gu(2)
        }
        spacing: units.gu(2)

        // Temporary buttons: I just need toolbar icons...
        // And first time is much more faster than swipe to show toolbar.
        Button {
            id: btnAppendFeed

            text: i18n.tr("Append")
            width: units.gu(15)

            onClicked: {
                PopupUtils.open(popoverSheetComponent, btnAppendFeed)
            }
        } // Button
    } // Row
} // Page
